package com.springinteg.aggregator;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.integration.channel.QueueChannel;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.GenericMessage;

import com.springinteg.aggregator.gateway.OrderGateway;
import com.springinteg.aggregator.model.Order;

public class OrderAggregatorTest {
	ApplicationContext context = null;
	
	@Before
	public void setUp() {
		context = new ClassPathXmlApplicationContext("spring-integ-context.xml");
	}
	

	@Before
	public void destroy() {
		context = null;
	}

	@Test
	public void testOrderAggregator() {
		OrderGateway paygate = context.getBean("ordergatewaybean", OrderGateway.class);
		Order p1 = new Order("SINGLE", 11);
		Order p2 = new Order("RECURRING", 25);
		Order p3 = new Order("SINGLE", 32);
		Order p4 = new Order("RECURRING", 15);
		QueueChannel orderChannel = context.getBean("orderOutChannel", QueueChannel.class);

		Message<Order> mp1 = new GenericMessage<Order>(p1);
		paygate.process(mp1);
		Message<List<Order>> processedOrders = (Message<List<Order>>) orderChannel.receive(10000);
		assertNull(processedOrders);
		Message<Order> mp2 = new GenericMessage<Order>(p2);
		Message<Order> mp3 = new GenericMessage<Order>(p3);
		Message<Order> mp4 = new GenericMessage<Order>(p4);

		paygate.process(mp2);
		paygate.process(mp3);
		paygate.process(mp4);
		processedOrders = (Message<List<Order>>) orderChannel.receive(60000);
		verifyAggregOrderTypes(processedOrders);

	}
	
	private void verifyAggregOrderTypes(Message<List<Order>> reply) {
		// TODO Auto-generated method stub
		List<Order> orders = reply.getPayload();
		Order firstOrder = orders.get(0);
		for (int i = 1; i < orders.size(); i++) {
			assertEquals(firstOrder.getType(), orders.get(i).getType());
		}
	}

}
